@@ -96,6 +96,11 @@ static inline void a_spin()
 	__asm__ __volatile__( "pause" : : : "memory" );
 }
 
+static inline void a_barrier()
+{
+	__asm__ __volatile__( "" : : : "memory" );
+}
+
 static inline void a_crash()
 {
 	__asm__ __volatile__( "hlt" : : : "memory" );
