@@ -95,8 +95,11 @@ static inline void a_store(volatile int *p, int x)
 		: "=m"(*p) : "r"(x) : "memory" );
 }
 
-static inline void a_spin()
+#define a_spin a_barrier
+
+static inline void a_barrier()
 {
+	a_cas(&(int){0}, 0, 0);
 }
 
 static inline void a_crash()
