@@ -3,6 +3,9 @@
 ((union { long long ll; long l[2]; }){ .ll = x }).l[1]
 #define __SYSCALL_LL_O(x) 0, __SYSCALL_LL_E((x))
 
+#ifdef SHARED
+__attribute__((visibility("hidden")))
+#endif
 long (__syscall)(long, ...);
 
 #define SYSCALL_RLIM_INFINITY (-1UL/2)
@@ -60,8 +63,9 @@ static inline long __syscall2(long n, long a, long b)
 		: "$1", "$3", "$8", "$9", "$10", "$11", "$12", "$13",
 		  "$14", "$15", "$24", "$25", "hi", "lo", "memory");
 	if (r7) return -r2;
+	long ret = r2;
 	if (n == SYS_stat64 || n == SYS_fstat64 || n == SYS_lstat64) __stat_fix(b);
-	return r2;
+	return ret;
 }
 
 static inline long __syscall3(long n, long a, long b, long c)
@@ -78,8 +82,9 @@ static inline long __syscall3(long n, long a, long b, long c)
 		: "$1", "$3", "$8", "$9", "$10", "$11", "$12", "$13",
 		  "$14", "$15", "$24", "$25", "hi", "lo", "memory");
 	if (r7) return -r2;
+	long ret = r2;
 	if (n == SYS_stat64 || n == SYS_fstat64 || n == SYS_lstat64) __stat_fix(b);
-	return r2;
+	return ret;
 }
 
 static inline long __syscall4(long n, long a, long b, long c, long d)
@@ -96,9 +101,10 @@ static inline long __syscall4(long n, long a, long b, long c, long d)
 		: "$1", "$3", "$8", "$9", "$10", "$11", "$12", "$13",
 		  "$14", "$15", "$24", "$25", "hi", "lo", "memory");
 	if (r7) return -r2;
+	long ret = r2;
 	if (n == SYS_stat64 || n == SYS_fstat64 || n == SYS_lstat64) __stat_fix(b);
 	if (n == SYS_fstatat) __stat_fix(c);
-	return r2;
+	return ret;
 }
 
 #else
