@@ -55,7 +55,4 @@
 #define MADV_DODUMP      17
 #define MADV_HWPOISON    100
 #define MADV_SOFT_OFFLINE 101
-
-#define MREMAP_MAYMOVE  1
-#define MREMAP_FIXED    2
 #endif
