@@ -374,6 +374,8 @@
 #define __NR_seccomp               358
 #define __NR_getrandom             359
 #define __NR_memfd_create          360
+#define __NR_bpf                   361
+#define __NR_execveat              362
 
 /*
  * repeated with SYS prefix
@@ -754,3 +756,5 @@
 #define SYS_seccomp               358
 #define SYS_getrandom             359
 #define SYS_memfd_create          360
+#define SYS_bpf                   361
+#define SYS_execveat              362
