@@ -1,5 +1,7 @@
 #include <features.h>
 
+#define START "_start"
+
 #include "crt_arch.h"
 
 int main();
@@ -8,7 +10,7 @@ void _fini() __attribute__((weak));
 _Noreturn int __libc_start_main(int (*)(), int, char **,
 	void (*)(), void(*)(), void(*)());
 
-void __cstart(long *p)
+void _start_c(long *p)
 {
 	int argc = p[0];
 	char **argv = (void *)(p+1);
