@@ -5,19 +5,22 @@
 extern "C" {
 #endif
 
+#include <features.h>
 #include <stdint.h>
 
-struct udphdr {
-	uint16_t source;
-	uint16_t dest;
-	uint16_t len;
-	uint16_t check;
-};
-
+#ifdef _GNU_SOURCE
 #define uh_sport source
 #define uh_dport dest
 #define uh_ulen len
 #define uh_sum check
+#endif
+
+struct udphdr {
+	uint16_t uh_sport;
+	uint16_t uh_dport;
+	uint16_t uh_ulen;
+	uint16_t uh_sum;
+};
 
 #define UDP_CORK	1
 #define UDP_ENCAP	100
