@@ -48,7 +48,7 @@ struct inotify_event {
 int inotify_init(void);
 int inotify_init1(int);
 int inotify_add_watch(int, const char *, uint32_t);
-int inotify_rm_watch(int, uint32_t);
+int inotify_rm_watch(int, int);
 
 #ifdef __cplusplus
 }
