@@ -1,4 +1,4 @@
-#include <stdlib.h>
+#include <stddef.h>
 
 size_t __ctype_get_mb_cur_max()
 {
