@@ -12,7 +12,7 @@ static struct fl
 	void *a[COUNT];
 } builtin, *head;
 
-static int lock[2];
+static volatile int lock[2];
 
 void __funcs_on_exit()
 {
