@@ -12,7 +12,7 @@ void *__simple_malloc(size_t n)
 {
 	static uintptr_t cur, brk;
 	uintptr_t base, new;
-	static int lock[2];
+	static volatile int lock[2];
 	size_t align=1;
 
 	if (!n) n++;
