@@ -24,12 +24,11 @@ int __fpclassifyl(long double x)
 {
 	union ldshape u = {x};
 	int e = u.i.se & 0x7fff;
+	u.i.se = 0;
 	if (!e)
 		return u.i2.lo | u.i2.hi ? FP_SUBNORMAL : FP_ZERO;
-	if (e == 0x7fff) {
-		u.i.se = 0;
+	if (e == 0x7fff)
 		return u.i2.lo | u.i2.hi ? FP_NAN : FP_INFINITE;
-	}
 	return FP_NORMAL;
 }
 #endif
