@@ -12,7 +12,7 @@ struct address {
 
 struct service {
 	uint16_t port;
-	char proto;
+	unsigned char proto, socktype;
 };
 
 /* The limit of 48 results is a non-sharp bound on the number of addresses
@@ -21,7 +21,7 @@ struct service {
 #define MAXADDRS 48
 #define MAXSERVS 2
 
-int __lookup_serv(struct service buf[static MAXSERVS], const char *name, int proto, int flags);
+int __lookup_serv(struct service buf[static MAXSERVS], const char *name, int proto, int socktype, int flags);
 int __lookup_name(struct address buf[static MAXADDRS], char canon[static 256], const char *name, int family, int flags);
 int __lookup_ipliteral(struct address buf[static 1], const char *name, int family);
 
