@@ -22,10 +22,11 @@ pid_t fork(void)
 #else
 	ret = syscall(SYS_clone, SIGCHLD, 0);
 #endif
-	if (libc.has_thread_pointer && !ret) {
+	if (!ret) {
 		pthread_t self = __pthread_self();
 		self->tid = __syscall(SYS_gettid);
-		memset(&self->robust_list, 0, sizeof self->robust_list);
+		self->robust_list.off = 0;
+		self->robust_list.pending = 0;
 		libc.threads_minus_1 = 0;
 	}
 	__restore_sigs(&set);
