@@ -1,6 +1,11 @@
 #include "pthread_impl.h"
 #include "syscall.h"
 
+#ifdef SHARED
+__attribute__((__visibility__("hidden")))
+#endif
+long __syscall_cp_c();
+
 static long sccp(syscall_arg_t nr,
                  syscall_arg_t u, syscall_arg_t v, syscall_arg_t w,
                  syscall_arg_t x, syscall_arg_t y, syscall_arg_t z)
